# -*- coding: utf-8 -*-

'''
Advanced Interface Switcher
2010-2012(c) Kirill Plyashkevich <ru.spirit@gmail.com>
Abstract interface object
'''

from __future__ import with_statement
import gtk, hildon, dbus, threading
from advpowcommon.ui.Banners import HildonBanner
from advpowcommon.ui.Animation import DITPBCachedAnimator
from advpowcommon.ui.ToggleControl import ToggleControlMaemo5
from advpowcommon.util.execn import *
import advifswlib.util.cacher as cacher
import advifswlib.util.config as config
import advifswlib.util.locale as locale

__all__ = ['InterfaceState', 'AbstractInterface']

class InterfaceState():
  off, on, connected, connecting, disconnecting = range(5)

class AbstractInterface():

  __interface_check = 'ps aux | grep -m 1 \'%s\' | grep -v \'grep\' | awk {\'print $3\'}'

  def __init__(self, bus=None, cb_status_area=None, 
               cb_show_settings_dialog=None):
    self.__running = False
    self.__processing = threading.Lock()
    self.__bus = bus
    self.__set_status_area_icon = cb_status_area
    self.__show_settings_dialog = cb_show_settings_dialog
    self.__interfaces = {}
    self.__lcl = locale.get()
    self.__config = config.get()
    self.__button = None
    self.interfaces = {}
    self.network = None
    self.state = {}

    self.force_startup_mode = False
    self.init_state()
    self.__name = self.state.pop('name') if 'name' in self.state else 'IFace'
    self.__config_path = 'settings/%s' % (self.__name.lower())
    self.__smi_cacher = cacher.get()
    self.__create_button()
    self.__switch_dialog = None
    self.state_control = None
    self.mode_control = None
    self.set_config_option('mode-on-startup', self.get_modes())
    self.__startup()
    start_state = int(self.get_config_value('enable-on-startup'))
    self.__switch_state(start_state)
    if start_state or self.force_startup_mode:
      self.__switch_mode(self.get_modes().index(self.get_config_value('mode-on-startup')), start_state == InterfaceState.off)
    self.__running = True
    #print self.__name, self.state['value'], self.state['mode'], self.state['strength']
    #gobject.timeout_add(5000, self.__show_switch_dialog)
    #self.__show_switch_dialog()

  def is_running(self):
    with self.__processing:
      return self.__running

  def __startup(self):
    self.__init_interfaces()
    self.__get_status()
    self.__init_signals()
    self.post_init()

  def __create_button(self):
    with self.__processing:
      if not self.__button:
        self.__button = hildon.Button(
          gtk.HILDON_SIZE_AUTO_WIDTH | gtk.HILDON_SIZE_FINGER_HEIGHT, 
          hildon.BUTTON_ARRANGEMENT_VERTICAL)
        self.__button.set_image(gtk.Image())
        self.__button.set_alignment(0.5,0.5,1,1)
        self.__button.set_image_alignment(0.5,0.5)
        self.__button.set_title_alignment(0,0.5)
        self.__button.set_value_alignment(0,0.5)
        self.__button.connect('clicked', self.__button_clicked)
        self.__button.tap_and_hold_setup(None, None, 0)
        self.__button.connect('tap-and-hold', self.__button_tap_and_hold)
        self.__button.connect('destroy', self.__destroy)

  def init_state(self):
    pass

  def post_init(self):
    pass

  def turn_on(self):
    pass

  def turn_off(self):
    pass

  def get_name(self):
    return self.__name

  def get_button(self):
    return self.__button

  def get_bus(self):
    return self.__bus

  def get_modes(self):
    return list(self.state['modes'])

  def get_network_name(self):
    return self.network

  def set_config_option(self, key, values):
    self.__config.set_option('%s/%s' % (self.__config_path, key), values)

  def get_config_value(self, key):
    return self.__config.get_value('%s/%s' % (self.__config_path, key))

  def set_config_value(self, key, value, save=False):
    return self.__config.set_value('%s/%s' % (self.__config_path, key), value, save)

  def destroy(self, button=None):
    with self.__processing:
      if button or not self.__button:
        self.__destroy()
      else:
        self.__button.destroy()

  def __destroy(self, button=None):
    if hasattr(self, 'state'):
      if self.__switch_dialog:
        self.__switch_dialog.destroy()
        self.__switch_dialog = None
      self.__destroy_signals()
      self.__destroy_interfaces()
      self.__button = None
      self.__set_status_area_icon(self.__name, None, False)

  def __init_signals(self):
    for key in self.state['dbus']:
      self.init_signal(key)

  def init_signal(self, x, force=False):
    try:
      if self.get_config_value('show-signal-%s' % x) != False:
        if not 'match' in self.state['dbus'][x] or force:
          def callback(*args, **kwargs):
            self.__received_signal(x, *args, **kwargs)
          setattr(self, 'received_%s' % x, callback)
          self.state['dbus'][x]['match'] = self.__bus\
            .add_signal_receiver(getattr(self, 'received_%s' % x),
                                 **self.state['dbus'][x])
          return True
    except:
      print '%s | signal \'%s\' failed' % (self.__name, x)
    return False

  def reinit_signal(self, x):
    self.destroy_signal(x)
    self.init_signal(x)

  def __destroy_signals(self):
    for key in self.state['dbus']:
      self.destroy_signal(key)

  def destroy_signal(self, x):
    if 'match' in self.state['dbus'][x]:
      receiver_match = self.state['dbus'][x].pop('match')
      #self.__bus.clean_up_signal_match(receiver_match)
      self.__bus.remove_signal_receiver(receiver_match,**self.state['dbus'][x])
      receiver_match.remove()
      delattr(self, 'received_%s' % x)
      return True
    return False

  def __set_value(self, value):
    update = value != None and value != self.state['value']
    if update:
      self.state['value'] = value
      if self.state_control:
        self.state_control.set_active(value)
    return update

  def __set_mode(self, value):
    update = value != None and value != self.state['mode']
    if update:
      self.state['mode'] = value
      if self.mode_control:
        self.mode_control.set_active(value)
    return update

  def __set_strength(self, value):
    update = value != None and value != self.state['strength']
    if update:
      self.state['strength'] = value
    return update

  def __received_signal(self, key, *args, **kwargs):
    #try: 
    value, mode, strength = getattr(self, 'process_signal_%s' % key)(*args, **kwargs)
    #except Exception, e: print 'Error: ', e

    updated_value = self.__set_value(value)
    updated_mode = self.__set_mode(mode)
    updated_strength = self.__set_strength(strength)
    if updated_value or updated_strength:
      self.__update_ui()
    if updated_value or updated_mode:
      self.__show_banner()
    #print self.__name, value, mode, strength, updated_value, updated_mode, updated_strength

  def __init_interfaces(self):
    for x in self.interfaces:
      self.__init_interface(x)

  def __init_interface(self, x, force=False):
    try:
      if x in self.interfaces and all(self.interfaces[x][:3]):
        loaded = get_shell_command_output(self.__interface_check % 
                                            (self.interfaces[x][3])) != '' \
                   if self.interfaces[x][3] and not force \
                   else True
        if loaded:
          kwargs = {'introspect': self.interfaces[x][4],
                    'follow_name_owner_changes': True}
          self.__interfaces[x] = dbus.Interface(\
            self.get_bus().get_object(*self.interfaces[x][:2],**kwargs),
            self.interfaces[x][2])
        return loaded
    except Exception, e:
      if x in self.__interfaces:
        self.__interfaces.pop(x)
      print e
    return False

  def reinit_interface(self, x, force=False):
    self.__destroy_interface(x)
    self.__init_interface(x, force)

  def get_interface(self, x):
    try:
      if not x in self.__interfaces:
        self.__init_interface(x)
      return self.__interfaces[x]
    except:
      return None

  def __destroy_interfaces(self):
    for x in self.interfaces:
      self.__destroy_interface(x)

  def __destroy_interface(self, x):
    try:
      if x in self.__interfaces:
        self.__interfaces.pop(x)
      return True
    except:
      return False

  def __get_status(self):
    self.state['value'], self.state['mode'], self.state['strength'] = self.get_status()
    self.__update_ui()

  def update(self):
    self.__update_ui()

  def __update_ui(self):
    value = self.state['value']
    strength = self.state['strength']
    image = self.state['images_st'][strength] \
      if value == InterfaceState.connected and strength > 0 \
      else self.state['images'][value]
    if self.__button:
      self.__set_si(image)
      self.__button.set_value( \
        self.network \
        if value == InterfaceState.connected and self.get_config_value('show-network-in-sm') and self.network \
        else self.__lcl.get(self.state['text'][value])
      )

  def __button_clicked(self, button=None, user_data=None):
    return self.__switch(self.get_config_value('show-state-dialog'))

  def __button_tap_and_hold(self, button=None, user_data=None):
    return self.__switch(not(self.get_config_value('show-state-dialog')))

  def __switch(self, show_dialog):
    self.__button.get_toplevel().hide()
    if show_dialog:
      gobject.idle_add(self.__show_switch_dialog)
    else:
      self.__switch_state(int(self.state['value'] == InterfaceState.off))
    return True

  def __show_switch_dialog(self):
    if self.__button:
      if not self.__switch_dialog:
        self.__switch_dialog = gtk.Dialog('%s' % (self.__name), None,#self.__button.window, 
                                 gtk.DIALOG_NO_SEPARATOR,
                                 (gtk.STOCK_PREFERENCES, gtk.RESPONSE_OK))
        self.state_control = ToggleControlMaemo5(self.__get_state_controls())
        self.state_control.set_active(self.__get_state_control_value())
        self.__switch_dialog.vbox.pack_start(self.state_control.get_box(), True)
        height = self.state_control.get_height()

        if len(self.state['modes']) > 1:
          self.mode_control = ToggleControlMaemo5(self.__get_mode_controls())
          self.mode_control.set_active(self.state['mode'])
          self.__switch_dialog.vbox.pack_start(self.mode_control.get_box(), True)
          height += self.mode_control.get_height()

        settings_button = self.__switch_dialog.action_area.get_children()[0]
        settings_button.connect('clicked', self.__settings_button_clicked)

      self.__switch_dialog.show_all()
      response = self.__switch_dialog.run()
      if self.__switch_dialog:
        self.__switch_dialog.hide()
    return False

  def __settings_button_clicked(self, button=None, user_data=None):
    self.__switch_dialog.hide()
    self.__show_settings_dialog()
    self.__show_switch_dialog()

  def __get_state_control_value(self):
    value = self.state['value']
    if value == InterfaceState.connecting:
      return InterfaceState.on
    elif value == InterfaceState.disconnecting:
      #return InterfaceState.connected
      return InterfaceState.on
    elif value == InterfaceState.connected:
      return InterfaceState.on
    return value

  def __get_state_controls(self):
    texts = map(self.__lcl.get, list(self.state['text'][:2]))
    return zip(texts, [self.__turn_state for t in texts])

  def __get_mode_controls(self):
    texts = map(self.__lcl.get_item, list(self.state['modes']))
    return zip(texts, [self.__turn_mode for t in texts])

  def __turn_state(self, widget=None, control=None, cstate=None):
    if widget.get_active():
      if not cstate:
        cstate = control.get_active()
      chstate = min(1, int(cstate))
      value = self.__get_state_control_value()
      if chstate != value:
          self.__switch_state(chstate)
      elif cstate != value:
        control.set_active(value)

  def __turn_mode(self, widget=None, control=None, cmode=None):
    if widget.get_active():
      if not(cmode):
        cmode = control.get_active()
      chmode = max(0, control.get_active())
      #if self.state['value'] != InterfaceState.off and chmode != self.state['mode']:
      if chmode != self.state['mode']:
          self.__switch_mode(chmode)
      elif cmode != self.state['mode']:
        control.set_active(self.state['mode'])

  def __switch_state(self, new_state):
    with self.__processing:
    #if self.__processing.acquire():
      cstate = min(1, self.state['value'])
      new_state = max(0, min(new_state, len(self.state['text']) - 1))
      if new_state != cstate:
        self.__button.set_value('...')
        try:
          if new_state:
            self.turn_on()
          else:
            self.turn_off()
        except:
          self.update()
      #else:
      #  self.__processing.release()

  def __switch_mode(self, new_mode, startup=False):
    with self.__processing:
    #if self.__processing.acquire():
      cmode = max(0, self.state['mode'])
      new_mode = max(0, min(new_mode, len(self.state['modes']) - 1))
      if new_mode != cmode:
        try:
          self.turn_mode(new_mode, startup)
        except:
          self.update()
      #else:
      #  self.__processing.release()

  def turn_mode(self, mode, startup):
    pass

  def __set_si(self, key):
    self.__set_smi(key)
    self.__set_sai(key)

  def __set_smi(self, key):
    if self.__button:
      #print self.__name, key, self.__smi_cacher.get(key) != None
      self.__button.get_image().set_from_pixbuf(self.__smi_cacher.get(key))

  def __set_sai(self, key):
    if any(self.state['show']):
     self.__set_status_area_icon(self.__name,
       *([key]
       if self.state['show'][self.state['value']] and self.get_config_value('show-sa-icon')
       else [None, False]))

  def __show_banner(self):
    windex = self.state['value']
    if self.get_config_value('show-popups') and self.state['banner'][windex]:
      text = '%s: %s' % \
             (self.__name, 
             self.network \
               if windex == InterfaceState.connected and self.network
               else self.__lcl.get(self.state['text'][windex]))
      if len(self.state['modes']) > 1:
        text = '%s (%s)' % \
               (text,
                self.__lcl.get_item(self.state['modes'][self.state['mode']]))
      HildonBanner.show_information(self.__button, text)

  def config_changed(self, key, old_value, new_value):
    cb = 'config_changed_%s' % key.replace('-','_')
    if hasattr(self, cb):
      getattr(self, cb)(old_value, new_value)

  def config_changed_enabled(self, old_value, new_value):
    if old_value:
      if self.__button:
        self.__button.destroy()
        self.__button = None
      else:
        self.destroy()
    elif not self.__button:
      self.__startup()
      self.__create_button()
